/*
    Spakes - a client/server Snakes-clone for up to 4 players
    Copyright (C) 2001, 2002, 2004  Markus Triska

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact me at triska@gmx.at
*/

#include <string>
#include <sstream>
#include <algorithm>

#include "spakes.h"
#ifdef WINDOWS
#include <winsock.h>
#include <time.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <sys/time.h>
#endif

#include "udpsock.h"
#include "protocol.h"

#include "mylog.h"
extern MyLogFile logfile;
#define bzero(a, b) memset(a, 0, b)

using namespace std;

UDPSocket::UDPSocket(string pip, unsigned short pport)
{
	sockaddr_in sock;
	ip = pip;
	port = pport;

	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	bzero(&sock, sizeof(sock));

	sock.sin_family = AF_INET;
	if (pport == 0) {
		// Let me be a server
		sock.sin_port = 0;
		sock.sin_addr.s_addr = htonl(INADDR_ANY);
		bind(sockfd, (struct sockaddr *) &sock, sizeof(sock));
		sockaddr_in myaddr;
#ifdef WINDOWS
		int len;
#else
		unsigned int len;
#endif
		len = sizeof(myaddr);
		getsockname(sockfd, (struct sockaddr*) &myaddr, &len);
		port = ntohs(myaddr.sin_port);
		logfile << "Creating UDP server on port: " << port << " (" << myaddr.sin_port << " in the structure)\r\n";
	} else {
		sock.sin_port = htons(pport);
		sock.sin_addr.s_addr = inet_addr(ip.c_str());
		connect(sockfd, (struct sockaddr *) &sock, sizeof(sock));
		logfile << "Connecting to UDP server on port: " << pport << " (that is: " << sock.sin_port << " in the structure)\r\n";
	}
	bufferempty = true;
}


UDPSocket::~UDPSocket()
{
	Discard();
	closesocket(sockfd);
	logfile << "I've deleted my socket!\r\n";
}


void UDPSocket::Discard()
{
	string discard;
	while(Receive(discard));
}


bool UDPSocket::Receive(string &toreceive)
{
	char maxpack[100];
	if (bufferempty) {
		timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 0;
		fd_set fset;
		FD_ZERO(&fset);
		FD_SET(sockfd, &fset);
		if (select(sockfd + 1, &fset, NULL, NULL, &tv) < 0) return false;
		if (!FD_ISSET(sockfd, &fset)) return false;
		int numread = recv(sockfd, maxpack, 99, 0);
		if (numread < 0) return false;
		bufferempty = false;

		maxpack[numread] = 0;
		toreceive = maxpack;
		logfile << "UDP server received: " << maxpack << "\n";

		packet = maxpack;
		lastdelim = packet.begin();
	}

	string::iterator delimpos = find(lastdelim, packet.end(), (char) DELIMINATER);


	toreceive = string(lastdelim, delimpos);

	lastdelim = delimpos + 1;
	if (lastdelim == packet.end()) bufferempty = true;


/*	// Acknowledge the packet
	string spack = maxpack;
	int seqno;
	istringstream pack(spack);
	pack >> seqno;
	logfile << "Received: " << pack.str() << endl;

	ostringstream ackpack;
	ackpack << seqno << ' ' << 13;
//	send(sockfd, ackpack.str().c_str(), ackpack.str().size(), 0);

	toreceive = pack.str(); */
	return true;
}

void UDPSocket::Send(string &tosend)
{
//	ostringstream realpack;
//	realpack << 15 << ' ' << tosend.c_str();
//	logfile << "Sending: " << realpack.str() << endl;
	send(sockfd, tosend.c_str(), tosend.size(), 0);
	logfile << "I've just sent " << tosend << " via UDP!\r\n";
	// Three chances for ACKs, 1 sec. wait
/*	for (int i = 0; i < 3; i++) {
		timval tv;
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		fd_set rset;
		FD_ZERO(&rset);
		FD_SET(sockfd, &rset);
		select(sockfd + 1, &rset, NULL, NULL, &tv);
		if (FD_ISSET(sockfd, rset) {

	} */
}


